@@ -23,6 +23,7 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#define register
 #include <Python.h>
 #include "py_funcs.h"
 #include "inifcns.h"
@@ -38,6 +39,7 @@
 #include "pseries.h"
 #include "utils.h"
 
+#include <utility>
 #include <vector>
 #include <stdexcept>
 #include <sstream>
@@ -45,7 +47,7 @@
 #include <memory>
 
 #if PY_MAJOR_VERSION > 2
-#define PyString_FromString PyBytes_FromString
+#define PyString_FromString PyUnicode_FromString
 #endif
 
 namespace GiNaC {
@@ -56,7 +58,7 @@ inline void py_error(const char* errmsg) {
 }
 
 // Creates the hypergeometric Python BuiltinFunction object
-ex _2F1(ex a, ex b, ex c, ex x)
+ex _2F1(const ex& a, const ex& b, const ex& c, ex x)
 {
         exvector avec, bvec;
         avec.push_back(a);
@@ -64,7 +66,7 @@ ex _2F1(ex a, ex b, ex c, ex x)
         bvec.push_back(c);
         PyObject *lista = py_funcs.exvector_to_PyTuple(avec);
         PyObject *listb = py_funcs.exvector_to_PyTuple(bvec);
-        PyObject *z = py_funcs.ex_to_pyExpression(x);
+        PyObject *z = py_funcs.ex_to_pyExpression(std::move(x));
 
         PyObject* m = PyImport_ImportModule("sage.functions.hypergeometric");
         if (m == nullptr)
