@@ -745,6 +745,10 @@ ex power::subs(const exmap & m, unsigned options) const
         if (!are_ex_trivially_equal(basis, subsed_basis)
 	 || !are_ex_trivially_equal(exponent, subsed_exponent)) {
 		ex p = power(subsed_basis, subsed_exponent);
+		if (!is_exactly_a<power>(p)) {
+		    // trac 30378 and 31530: do not over-substitute
+	        return p;
+	    }
                 ex t = ex_to<power>(p).subs_one_level(m, options);
                 if ((t-*this).is_zero())
                         return p;
@@ -920,10 +924,14 @@ ex power::expand(unsigned options) const
 			const ex& e = m.recombine_pair_to_ex(elem);
 			if (e.is_positive())
 				prodseq.push_back(pow(e, exponent).expand(options));
-			else if (e.info(info_flags::negative)) {
-				prodseq.push_back(pow(-e, exponent).expand(options));
-				possign = !possign;
-			} else
+// 			we delete the following 'else if' clause because it can lead to
+// 			an infinite loop (see sagemath :trac:`30688`)
+// 			TODO: find a bug-free treatment of negative factors
+// 			else if (e.info(info_flags::negative)) {
+// 				prodseq.push_back(pow(-e, exponent).expand(options));
+// 				possign = !possign;
+// 			}
+			else
 				powseq.push_back(elem);
 		}
 
