@@ -10,6 +10,7 @@
 #include <unordered_map>
 #include <unordered_set>
 #include <cstdlib>
+#include <utility>
 
 #include "inifcns.h"
 #include "ex.h"
@@ -72,15 +73,15 @@ static ex binomial_to_gamma(const function& f)
         const ex& a = f.op(0);
         const ex& k = f.op(1);
         if (is_exactly_a<numeric>(a)) {
-                numeric anum = ex_to<numeric>(a);
-                if (anum.info(info_flags::integer)
+                const numeric& anum = ex_to<numeric>(a);
+                if (anum.is_integer()
                     and anum.info(info_flags::negative))
                         return pow(_ex_1, k) * 
                                 (gamma(k - a) / (gamma(k+1) * (anum-*_num1_p).factorial()));
         }
         ex t = (k - a).expand();
         if (is_exactly_a<numeric>(t)
-            and ex_to<numeric>(t).info(info_flags::integer)
+            and ex_to<numeric>(t).is_integer()
             and ex_to<numeric>(t).info(info_flags::negative))
                 return _ex0;
 
@@ -99,7 +100,7 @@ static ex falling_factorial_to_gamma(const function& f)
 
 using tgfun_t = decltype(gamma_to_gamma);
 
-static bool has_suitable_form(ex the_ex)
+static bool has_suitable_form(const ex& the_ex)
 {
         static std::unordered_map<unsigned int,tgfun_t*> funcmap {{
           {factorial_SERIAL::serial, &factorial_to_gamma},
@@ -112,16 +113,16 @@ static bool has_suitable_form(ex the_ex)
         if (is_rational_linear(the_ex))
                 return true;
         if (is_exactly_a<power>(the_ex)) {
-                power pow = ex_to<power>(the_ex);
+                const power& pow = ex_to<power>(the_ex);
                 const ex& expo = pow.op(1);
                 if (is_exactly_a<numeric>(expo)
-                    and expo.info(info_flags::integer))
+                    and expo.is_integer())
                         return has_suitable_form(pow.op(0));
                 return (is_rational_linear(pow.op(0))
                      and is_rational_linear(pow.op(1)));
         }
         if (is_exactly_a<function>(the_ex)) {
-                function f = ex_to<function>(the_ex);
+                const function& f = ex_to<function>(the_ex);
                 if (funcmap.find(f.get_serial()) == funcmap.end())
                         return false;
                 for (unsigned int i=0; i<f.nops(); i++)
@@ -161,10 +162,10 @@ ex to_gamma(const ex& the_ex)
         if (is_rational_linear(the_ex))
                 return the_ex;
         if (is_exactly_a<power>(the_ex)) {
-                power pow = ex_to<power>(the_ex);
+                const power& pow = ex_to<power>(the_ex);
                 const ex& expo = pow.op(1);
                 if (is_exactly_a<numeric>(expo)
-                    and expo.info(info_flags::integer))
+                    and expo.is_integer())
                         return power(to_gamma(pow.op(0)), expo);
                 return the_ex;
         }
@@ -239,10 +240,10 @@ static ex combine_powers(const ex& the_ex)
 }
 
 using ex_intset_map = std::map<GiNaC::ex, std::unordered_set<int>, GiNaC::ex_is_less>;
-static void collect_gamma_args(ex the_ex, ex_intset_map& map)
+static void collect_gamma_args(const ex& the_ex, ex_intset_map& map)
 {
         if (is_exactly_a<function>(the_ex)) {
-                function f = ex_to<function>(the_ex);
+                const function& f = ex_to<function>(the_ex);
                 if (f.get_serial() == gamma_SERIAL::serial) {
                         ex arg = f.op(0).expand();
                         if (is_exactly_a<numeric>(arg))
@@ -274,7 +275,7 @@ static void collect_gamma_args(ex the_ex, ex_intset_map& map)
                         collect_gamma_args(f.op(i), map);
         }
         else if (is_exactly_a<power>(the_ex)) {
-                power pow = ex_to<power>(the_ex);
+                const power& pow = ex_to<power>(the_ex);
                 collect_gamma_args(pow.op(0), map);
                 collect_gamma_args(pow.op(1), map);
         }
@@ -310,11 +311,10 @@ ex gamma_normalize(ex the_ex)
         bool res = factor(subsed, res_ex);
         if (res)
                 return res_ex;
-        else
-                return subsed;
+        return subsed;
 }
 
-ex hypersimp(const ex& e, const ex& k)
+ex hypersimp(ex e, ex k)
 // See Algorithm 2.1 in the Koepf reference
 {
         ex f = e.expand();
@@ -348,8 +348,9 @@ static ex binomial_poly(const exvector& syms, const ex& var)
         for (unsigned int row=0; row<n; ++row) {
                 const ex& v = power(var, row);
                 for (unsigned int col=0; col<n; ++col) {
-                        if (row+col < n)
+                        if (row+col < n) {
                                 res += v * binomial(row+col, col) * syms[n-col-row-1];
+                        }
                 }
         }
         return res;
@@ -377,7 +378,7 @@ static matrix solve_system(ex mpoly,
                 const ex& expo = pair.second;
                 if (not is_exactly_a<numeric>(expo))
                         throw std::runtime_error("can't happen in solve_system()");
-                numeric nume = ex_to<numeric>(expo);
+                const numeric& nume = ex_to<numeric>(expo);
                 if (not nume.is_mpz() and not nume.is_long())
                         throw std::runtime_error("can't happen in solve_system()");
                 int e = nume.to_int();
@@ -408,7 +409,7 @@ static bool check_root(const matrix& M,
         map[v] = ex(numeric(root));
         std::srand(std::time(0));
         for (int i=0; i<NCHECKS; ++i) {
-                for (const auto s : sy)
+                for (const auto& s : sy)
                         if (not v.is_equal(s))
                                 map[s] = numeric((std::rand()%9)+1);
                 for (int l = 0; l<msize; ++l)
@@ -523,7 +524,7 @@ ex gosper_term(ex e, ex n)
                 D.insert(K - N + 1);
                 ex t = (B.coeff(n,N-1) - A.coeff(n,N-1)) / A.lcoeff(n);
                 if (is_exactly_a<numeric>(t)
-                    and ex_to<numeric>(t).info(info_flags::integer)
+                    and ex_to<numeric>(t).is_integer()
                     and ex_to<numeric>(t) >= *_num0_p)
                                 D.insert(ex_to<numeric>(t).to_int());
         }
@@ -534,7 +535,7 @@ ex gosper_term(ex e, ex n)
                 throw gosper_domain_error();
         exvector syms;
         for (int i=0; i<d+1; ++i)
-                syms.push_back((new symbol)->setflag(status_flags::dynallocated));
+                syms.emplace_back((new symbol)->setflag(status_flags::dynallocated));
         ex xshifted = binomial_poly(syms, n);
         ex x = diagonal_poly(syms, n);
         ex H = A*xshifted - B*x -C;
@@ -544,7 +545,7 @@ ex gosper_term(ex e, ex n)
                 ex val = ex_to<numeric>(solution(i,0));
                 x = x.subs(sym == val);
         }
-        for (auto sym : syms)
+        for (const auto& sym : syms)
                 x = x.subs(sym == _ex0);
         return B*x / C;
 }
@@ -559,8 +560,7 @@ ex gosper_sum_definite(ex f, ex s, ex a, ex b, int* success)
                 bool changed = factor(t, res);
                 if (changed)
                         return res;
-                else
-                        return t;
+                return t;
         }
         catch (gosper_domain_error) {
                 *success = 0;
@@ -571,14 +571,13 @@ ex gosper_sum_definite(ex f, ex s, ex a, ex b, int* success)
 ex gosper_sum_indefinite(ex f, ex s, int* success)
 {
         try {
-                ex t = f*gosper_term(f, s);
+                ex t = f*gosper_term(f, std::move(s));
                 *success = 1;
                 ex res;
                 bool changed = factor(t, res);
                 if (changed)
                         return res;
-                else
-                        return t;
+                return t;
         }
         catch (gosper_domain_error) {
                 *success = 0;
